Class: Expressir::Model::RemarkInfo

Inherits:
Lutaml::Model::Serializable
  • Object
show all
Defined in:
lib/expressir/model/remark_info.rb

Overview

Represents a remark with its format and optional tag Supports both tail remarks (– …) and embedded remarks ((* … *)) Can include optional tags for associating remarks with specific items

Instance Method Summary collapse

Instance Method Details

#embedded?Boolean

Check if this is an embedded remark

Returns:

  • (Boolean)

    True if format is ‘embedded’



19
20
21
# File 'lib/expressir/model/remark_info.rb', line 19

def embedded?
  format == "embedded"
end

#tagged?Boolean

Check if this remark has a tag

Returns:

  • (Boolean)

    True if tag is present



25
26
27
# File 'lib/expressir/model/remark_info.rb', line 25

def tagged?
  !tag.nil? && !tag.empty?
end

#tail?Boolean

Check if this is a tail remark

Returns:

  • (Boolean)

    True if format is ‘tail’



13
14
15
# File 'lib/expressir/model/remark_info.rb', line 13

def tail?
  format == "tail"
end